<?php
/*************************************************************************************/
/*                                                                                   */
/*      Thelia - Plugin Forfait par zone                                		 	 */
/*                                                                                   */
/*      Copyright (c) Franck Allimant, 2010	                                     	 */
/*		email : thelia@allimant.org 	        	                             	 */
/*      web : http://www.allimant.org                                                */
/*                                                                                   */
/*      This program is free software; you can redistribute it and/or modify         */
/*      it under the terms of the GNU General Public License as published by         */
/*      the Free Software Foundation; either version 2 of the License, or            */
/*      (at your option) any later version.                                          */
/*                                                                                   */
/*      This program is distributed in the hope that it will be useful,              */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of               */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                */
/*      GNU General Public License for more details.                                 */
/*                                                                                   */
/*      You should have received a copy of the GNU General Public License            */
/*      along with this program; if not, write to the Free Software                  */
/*      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA    */
/*                                                                                   */
/*************************************************************************************/
include_once(realpath(dirname(__FILE__)) . "/../../../classes/PluginsTransports.class.php");
include_once(realpath(dirname(__FILE__)) . "/../../../classes/Variable.class.php");
include_once(realpath(dirname(__FILE__)) . "/../../../fonctions/divers.php");

class ForfaitZoneQuantite extends PluginsTransports
{

	const VERSION = '1.4.2';
	const VERSION_NUM = 14;
	const PREFIXE = 'fzq_';

	const MODULE_NAME = 'forfaitzonequantite';

	function ForfaitZoneQuantite()
    {
		$this->PluginsTransports(self::MODULE_NAME);
	}

    // Init departements
	function populate_depts()
	{
	    $query = "delete from fzq_departements";

	    $this->_query($query);

	    $values = "";

        if ($fp = fopen(realpath(dirname(__FILE__)) . "/departements.csv", "r"))
        {
            while ($line = fgets($fp))
            {
                list($num, $nom) = explode(";", $line);

                if ($num != "" && $nom != "")
                {
                    $nom = addslashes($nom);

                    if (strlen($num) < 2) $num = "0$num";

                    $values .= ($values == "" ? "" : ",") . "('$num', '$nom')";
                }
            }

            @fclose($fp);
        }

        if ($values != "")
        {
            $query = "insert into fzq_departements (id, nom) values ".$values;

            $result = $this->_query($query);
        }
	}

	function init()
    {
		$this->ajout_desc("Forfait par zone et quantité", "Forfait par zone et quantité", "", 1);

		$query = "CREATE TABLE IF NOT EXISTS fzq_departements (
		  id varchar(10) NOT NULL,
		  nom text NOT NULL,
		  PRIMARY KEY  (id)
		) AUTO_INCREMENT=1 ;";

		$result = $this->_query($query);

        $this->populate_depts($cnx);

		$query = "CREATE TABLE IF NOT EXISTS fzq_zones (
		  id int(11) NOT NULL auto_increment,
		  nom text NOT NULL,
		  PRIMARY KEY  (id)
		) AUTO_INCREMENT=1 ;";

		$result = $this->_query($query);

		$query = "CREATE TABLE IF NOT EXISTS fzq_def_zones (
		  id int(11) NOT NULL auto_increment,
		  id_zone int(11) NOT NULL,
		  departement varchar(10) NOT NULL,
		  PRIMARY KEY  (id),
		  INDEX (id_zone, departement)
		) AUTO_INCREMENT=1 ;";

		$result = $this->_query($query);

		$query = "CREATE TABLE IF NOT EXISTS fzq_data (
		  id int(11) NOT NULL auto_increment,
		  id_zone int(11) NOT NULL,
		  quantite_max float default 0,
		  forfait float,
		  surcout_unitaire float,
		  mode_surcout varchar(5),
		  PRIMARY KEY  (id),
          INDEX (id_zone, quantite_max)
		) AUTO_INCREMENT=1 ;";

		$result = $this->_query($query);

        // Update 1.1
        $this->_query("ALTER TABLE `fzq_departements` CHANGE `id` `id` VARCHAR(10) NOT NULL", false);
        $this->_query("ALTER TABLE `fzq_def_zones` CHANGE `departement` `departement` VARCHAR(10) NOT NULL", false);

        // Update 1.4
        $this->_query("ALTER TABLE `fzq_data` ADD `mode_surcout` varchar(5) NOT NULL", false);
        $this->_query("ALTER TABLE `fzq_data` DROP `quantite_min`", false);

        $this->create_variable('fzq_valeur_defaut', 0);
        $this->create_variable('fzq_mode_calcul', 'quantite');
        $this->create_variable('fzq_version', self::VERSION);
    }

    function trouver_tarif($quantite, $departement)
    {
		$query = "
            select d.*
            from
                fzq_data d, fzq_def_zones dz
            where
		         dz.departement = LEFT('$departement', LENGTH(dz.departement))
             and
                 d.id_zone = dz.id_zone
		     and
		         $quantite <= quantite_max
             order by
                 LENGTH(dz.departement) desc,
                 quantite_max asc
             limit 1
        ";

		$result = $this->_query($query);

        if ($result && $row = mysql_fetch_assoc($result))
        {
        	switch($row['mode_surcout'])
        	{
	        	case 'fix':
	        		$surplus = floatval($row['surcout_unitaire']);
	        	break;

	        	case 'pcent':
	        		$surplus = floatval($row['forfait']) * floatval($row['surcout_unitaire']) / 100;
				break;

	        	default:
	        		$surplus = 0;
        	}

            return round(floatval($row['forfait']) +  $quantite * $surplus, 2);
        }

        // Nous consulter -> tarif 0
        return floatval($this->get_variable('fzq_valeur_defaut'));
    }

    function action()
    {
    	global $res;

  		$res = str_replace('#FZQ_PORT_DEFAUT', floatval($this->get_variable('fzq_valeur_defaut')), $res);
    }

    function boucle($texte, $args)
    {
        $boucle = strtolower(lireTag($args, 'boucle'));

        switch($boucle)
        {
            case 'zones' :
                return $this->boucleZones($texte, $args);
            break;

            case 'departements' :
                return $this->boucleDepartements($texte, $args);
            break;

            case 'tarifs' :
                return $this->boucleTarifs($texte, $args);
            break;
        }
    }

    function boucleZones($texte, $args)
    {
        $id_zone = intval(lireTag($args, 'idzone'));

        $where = '';

        if ($id_zone != 0)
        {
            $where = " where id = $id_zone";
        }

        $infos = $this->query_to_array("select * from fzq_zones $where order by nom");

        $res = '';

        foreach($infos as $info)
        {
            $tmp = str_replace("#ID", $info['id'], $texte);
            $tmp = str_replace("#NOM", $info['nom'], $tmp);

            $res .= $tmp;
        }

        return $res;
    }

    function boucleDepartements($texte, $args)
    {
        $id_zone = intval(lireTag($args, 'idzone'));

        $where = '';

        if ($id_zone != 0)
        {
            $where = " dz.id_zone = $id_zone and";
        }

        $query = "select d.*
            from
                fzq_def_zones dz, fzq_departements d
            where
                 $where
		         d.id = dz.departement
             order by
                 d.id";

        $infos = $this->query_to_array($query);

        $res = '';

        foreach($infos as $info)
        {
            $tmp = str_replace("#NUMERO", $info['id'], $texte);
            $tmp = str_replace("#NOM", $info['nom'], $tmp);

            $res .= $tmp;
        }

        return $res;
    }

    function boucleTarifs($texte, $args)
    {
        $res = '';

        $id_zone = intval(lireTag($args, 'idzone'));
        $departement = lireTag($args, 'departement');
        $id_adresse = lireTag($args, 'adresse');
        $redir = intval(lireTag($args, 'rediriger'));

        $where = '';

        if ($id_adresse == '0')
        {
            $departement = $_SESSION['navig']->client->cpostal;
        }
        else if ($id_adresse != '')
        {
            $adr = new Adresse();

            if ($adr->charger(intval($id_adresse)))
            {
                $departement = $adr->cpostal;
            }
        }

        if ($departement != '')
        {
            $query = "SELECT *, z.nom as nom_zone
                FROM
                    fzq_departements d
                LEFT JOIN
                    fzq_def_zones as dz ON d.id=dz.departement
                LEFT JOIN
                    fzq_data as zd on dz.id_zone = zd.id_zone
                LEFT JOIN
                    fzq_zones AS z ON z.id = zd.id_zone
                where
                    d.id='$departement'
                 order by
                    quantite_max";
        }
        else if ($id_zone != 0)
        {
    		$query = "
                select *
                from
                    fzq_data
                where
                    id_zone = $id_zone
                order by
                    quantite_max";
        }
        else if ($redir != '')
        {
            header('Location: ' . $redir);
            exit();
        }

        $infos = $this->query_to_array($query);

        foreach($infos as $info)
        {
        	$tmp = $texte;

            $tmp = str_replace("#QUANTITEMAX", $info['quantite_max'], $tmp);
            $tmp = str_replace("#PRIX", $info['forfait'], $tmp);
            $tmp = str_replace("#SURCOUTUNITAIRE", $info['surcout_unitaire'], $tmp);
            $tmp = str_replace("#MODESURCOUT", $info['mode_surcout'], $tmp);
            $tmp = str_replace("#NOM_ZONE", trim($info['nom_zone']), $tmp);

            $res .= $tmp;
        }

        return $res;
    }

	function calcule()
    {
        $departement = strtoupper(substr($this->cpostal, 0, 2));

        $mode = $this->get_variable('fzq_mode_calcul');

        if ($mode == 'quantite')
        {
            return $this->trouver_tarif($this->nbart, $departement);
        }
        else if ($mode == 'poids')
        {
            return $this->trouver_tarif($this->poids, $departement);
        }
        else if ($mode == 'prix')
        {
            return $this->trouver_tarif($this->total, $departement);
        }
        else
        {
            die("Le mode de calcul du plugin Forfait Zone Quantité n'est pas défini. Indiquez le dans le Back Office.");
        }
	}

    //-- Outils ----------------------------------------------------------------

    public function get_variable($nom)
    {
        $var = new Variable();

        if ($var->charger($nom))
        {
            return $var->valeur;
        }

        return false;
    }

    protected function create_variable($nom, $valeur)
    {
        $variable = new Variable();

        $variable->nom = $nom;
        $variable->valeur = $valeur;
        $variable->protege = 1;
        $variable->cache = 1;

        $variable->add();
    }

    protected function delete_variable($nom)
    {
        $variable = new Variable();

        if ($variable->charger($nom)) $variable->delete();
    }

	//-- DB utils ----------------------------------------------------------------

	function escape($value)
    {
        if(get_magic_quotes_gpc()) $value = stripslashes($value);

        return mysql_real_escape_string($value);
    }

    function _query($query, $die = 1)
    {
        $result = mysql_query($query);

        if ($die && ! $result) die ("Erreur: $query<br>".mysql_error());

        return $result;
    }

    function query_to_obj($query)
    {
        $result = $this->_query($query, 0);

        if ($result)
        {
            return mysql_fetch_assoc($result);
        }

        return null;
    }

    function query_to_array($query)
    {
        $arr = array();

        $result = $this->_query($query, 0);

        while ($result && $row = mysql_fetch_assoc($result))
        {
            $arr[] = $row;
        }

        return $arr;
    }
}
?>
